package de.w665.biblenotes.rest.mappings;

import de.w665.biblenotes.db.entity.BibleReadingPlan;
import de.w665.biblenotes.db.entity.User;
import de.w665.biblenotes.db.repo.BibleReadingPlanRepository;
import de.w665.biblenotes.db.repo.UserRepository;
import de.w665.biblenotes.rest.SecureApiRestController;
import de.w665.biblenotes.rest.dto.BibleReadingPlanDTO;
import de.w665.biblenotes.rest.security.UserAuthenticationContext;
import jakarta.persistence.EntityManager;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import java.util.Optional;

@RestController
@RequestMapping("/secure/bible-reading-plan")
public class BibleReadingPlanMapping  {

    private final EntityManager entityManager;
    private final BibleReadingPlanRepository bibleReadingPlanRepository;

    public BibleReadingPlanMapping(UserRepository userRepository, EntityManager entityManager, BibleReadingPlanRepository bibleReadingPlanRepository) {
        this.entityManager = entityManager;
        this.bibleReadingPlanRepository = bibleReadingPlanRepository;
    }

    @GetMapping
    public ResponseEntity<Object> getBibleReadingPlans(@RequestParam(name = "id", required = true) Long id) {
        Optional<BibleReadingPlan> brp = bibleReadingPlanRepository.findById(id);
        if(brp.isEmpty()) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }

        BibleReadingPlanDTO bibleReadingPlanDTO = new BibleReadingPlanDTO();
        bibleReadingPlanDTO.setId(brp.get().getId());
        bibleReadingPlanDTO.setPlanName(brp.get().getPlanName());
        bibleReadingPlanDTO.setChapterPerDay(brp.get().getChapterPerDay());
        bibleReadingPlanDTO.setStartDate(brp.get().getStartDate());
        bibleReadingPlanDTO.setCreatorUserName(brp.get().getCreatorUser().getUsername());

        return new ResponseEntity<>(bibleReadingPlanDTO, HttpStatus.OK);
    }

    @PostMapping
    public ResponseEntity<Object> createBibleReadingPlan(@RequestBody BibleReadingPlanDTO bibleReadingPlanDTO) {
        Authentication auth = SecurityContextHolder.getContext().getAuthentication();

        // Construct entity
        BibleReadingPlan plan = new BibleReadingPlan();
        plan.setPlanName(bibleReadingPlanDTO.getPlanName());
        plan.setChapterPerDay(bibleReadingPlanDTO.getChapterPerDay());
        plan.setStartDate(bibleReadingPlanDTO.getStartDate());

        // Set user as creator in db entity
        User userProxy = entityManager.getReference(User.class, ((UserAuthenticationContext) auth.getPrincipal()).getUserId());
        plan.setCreatorUser(userProxy);
        BibleReadingPlan createdPlan = bibleReadingPlanRepository.save(plan);

        // Construct response (we do this because otherwise the user object would be serialized (which would fail because it's a proxy + i addition we don't want to send the users credentials))
        BibleReadingPlanDTO responseDTO = new BibleReadingPlanDTO();
        responseDTO.setPlanName(createdPlan.getPlanName());
        responseDTO.setChapterPerDay(createdPlan.getChapterPerDay());
        responseDTO.setStartDate(createdPlan.getStartDate());
        responseDTO.setCreatorUserName(((UserAuthenticationContext) auth.getPrincipal()).getUsername());
        responseDTO.setId(createdPlan.getId());

        return new ResponseEntity<>(responseDTO, HttpStatus.OK);
    }
}
